@@ -67,11 +67,12 @@ class BlockTests(testing.TestCase):
 
     def test_volume_detail(self):
         result = self.run_command(['block', 'volume-detail', '1234'])
+
         self.assert_no_fail(result)
         self.assertEqual({
             'Username': 'username',
             'LUN Id': '2',
-            'Endurance Tier': '2 IOPS per GB',
+            'Endurance Tier': 'READHEAVY_TIER',
             'IOPs': 1000,
             'Snapshot Capacity (GB)': '10',
             'Snapshot Used (Bytes)': 1024,
@@ -94,7 +95,13 @@ class BlockTests(testing.TestCase):
                 {'Replicant ID': 'Target IP', '1785': '10.3.177.84'},
                 {'Replicant ID': 'Data Center', '1785': 'dal01'},
                 {'Replicant ID': 'Schedule', '1785': 'REPLICATION_DAILY'},
-            ]]
+            ]],
+            'Duplicate Volume Properties': [
+                {'Original Volume Name': 'Original Volume Size',
+                 'test-origin-volume-name': '20'},
+                {'Original Volume Name': 'Original Snapshot Name',
+                 'test-origin-volume-name': 'test-origin-snapshot-name'}
+            ]
         }, json.loads(result.output))
 
     def test_volume_list(self):
@@ -217,6 +224,30 @@ class BlockTests(testing.TestCase):
                          'Order could not be placed! Please verify '
                          'your options and try again.\n')
 
+    @mock.patch('SoftLayer.BlockStorageManager.order_block_volume')
+    def test_volume_order_performance_manager_error(self, order_mock):
+        order_mock.side_effect = ValueError('failure!')
+
+        result = self.run_command(['block', 'volume-order',
+                                   '--storage-type=performance', '--size=20',
+                                   '--iops=100', '--os-type=linux',
+                                   '--location=dal05'])
+
+        self.assertEqual(2, result.exit_code)
+        self.assertEqual('Argument Error: failure!', result.exception.message)
+
+    @mock.patch('SoftLayer.BlockStorageManager.order_block_volume')
+    def test_volume_order_endurance_manager_error(self, order_mock):
+        order_mock.side_effect = ValueError('failure!')
+
+        result = self.run_command(['block', 'volume-order',
+                                   '--storage-type=endurance', '--size=20',
+                                   '--tier=0.25', '--os-type=linux',
+                                   '--location=dal05'])
+
+        self.assertEqual(2, result.exit_code)
+        self.assertEqual('Argument Error: failure!', result.exception.message)
+
     def test_enable_snapshots(self):
         result = self.run_command(['block', 'snapshot-enable', '12345678',
                                    '--schedule-type=HOURLY', '--minute=10',
@@ -320,6 +351,54 @@ class BlockTests(testing.TestCase):
         self.assertEqual('Failover to replicant is now in progress.\n',
                          result.output)
 
+    def test_replication_locations(self):
+        result = self.run_command(['block', 'replica-locations', '1234'])
+        self.assert_no_fail(result)
+        self.assertEqual(
+            {
+                '12345': 'Dallas 05',
+            },
+            json.loads(result.output))
+
+    @mock.patch('SoftLayer.BlockStorageManager.get_replication_locations')
+    def test_replication_locations_unsuccessful(self, locations_mock):
+        locations_mock.return_value = False
+        result = self.run_command(['block', 'replica-locations', '1234'])
+        self.assertEqual('No data centers compatible for replication.\n',
+                         result.output)
+
+    def test_replication_partners(self):
+        result = self.run_command(['block', 'replica-partners', '1234'])
+        self.assert_no_fail(result)
+        self.assertEqual([
+            {
+                'ID': 1784,
+                'Account ID': 3000,
+                'Capacity (GB)': 20,
+                'Host ID': None,
+                'Guest ID': None,
+                'Hardware ID': None,
+                'Username': 'TEST_REP_1',
+            },
+            {
+                'ID': 1785,
+                'Account ID': 3001,
+                'Host ID': None,
+                'Guest ID': None,
+                'Hardware ID': None,
+                'Capacity (GB)': 20,
+                'Username': 'TEST_REP_2',
+            }],
+            json.loads(result.output))
+
+    @mock.patch('SoftLayer.BlockStorageManager.get_replication_partners')
+    def test_replication_partners_unsuccessful(self, partners_mock):
+        partners_mock.return_value = False
+        result = self.run_command(['block', 'replica-partners', '1234'])
+        self.assertEqual(
+            'There are no replication partners for the given volume.\n',
+            result.output)
+
     @mock.patch('SoftLayer.BlockStorageManager.failover_to_replicant')
     def test_replicant_failover_unsuccessful(self, failover_mock):
         failover_mock.return_value = False
@@ -390,3 +469,45 @@ class BlockTests(testing.TestCase):
                          ' > 20 GB Storage Space\n'
                          ' > 10 GB Storage Space (Snapshot Space)\n'
                          ' > 20 GB Storage Space Replicant of: TEST\n')
+
+    @mock.patch('SoftLayer.BlockStorageManager.order_duplicate_volume')
+    def test_duplicate_order_exception_caught(self, order_mock):
+        order_mock.side_effect = ValueError('order attempt failed, oh noooo!')
+
+        result = self.run_command(['block', 'volume-duplicate', '102'])
+
+        self.assertEqual(2, result.exit_code)
+        self.assertEqual('Argument Error: order attempt failed, oh noooo!',
+                         result.exception.message)
+
+    @mock.patch('SoftLayer.BlockStorageManager.order_duplicate_volume')
+    def test_duplicate_order_order_not_placed(self, order_mock):
+        order_mock.return_value = {}
+
+        result = self.run_command(['block', 'volume-duplicate', '102',
+                                   '--duplicate-iops=1400'])
+
+        self.assert_no_fail(result)
+        self.assertEqual(result.output,
+                         'Order could not be placed! Please verify '
+                         'your options and try again.\n')
+
+    @mock.patch('SoftLayer.BlockStorageManager.order_duplicate_volume')
+    def test_duplicate_order(self, order_mock):
+        order_mock.return_value = {
+            'placedOrder': {
+                'id': 24601,
+                'items': [{'description': 'Storage as a Service'}]
+            }
+        }
+
+        result = self.run_command(['block', 'volume-duplicate', '102',
+                                   '--origin-snapshot-id=470',
+                                   '--duplicate-size=250',
+                                   '--duplicate-tier=2',
+                                   '--duplicate-snapshot-size=20'])
+
+        self.assert_no_fail(result)
+        self.assertEqual(result.output,
+                         'Order #24601 placed successfully!\n'
+                         ' > Storage as a Service\n')
