@@ -105,10 +105,37 @@ class FileTests(testing.TestCase):
 
         self.assertEqual(fixtures.SoftLayer_Network_Storage.getObject, result)
 
+        expected_mask = 'id,'\
+                        'username,'\
+                        'password,'\
+                        'capacityGb,'\
+                        'bytesUsed,'\
+                        'snapshotCapacityGb,'\
+                        'parentVolume.snapshotSizeBytes,'\
+                        'storageType.keyName,'\
+                        'serviceResource.datacenter[name],'\
+                        'serviceResourceBackendIpAddress,'\
+                        'fileNetworkMountAddress,'\
+                        'storageTierLevel,'\
+                        'iops,'\
+                        'lunId,'\
+                        'originalVolumeName,'\
+                        'originalSnapshotName,'\
+                        'originalVolumeSize,'\
+                        'activeTransactionCount,'\
+                        'activeTransactions.transactionStatus[friendlyName],'\
+                        'replicationPartnerCount,'\
+                        'replicationStatus,'\
+                        'replicationPartners[id,username,'\
+                        'serviceResourceBackendIpAddress,'\
+                        'serviceResource[datacenter[name]],'\
+                        'replicationSchedule[type[keyname]]]'
+
         self.assert_called_with(
             'SoftLayer_Network_Storage',
             'getObject',
-            identifier=100)
+            identifier=100,
+            mask='mask[%s]' % expected_mask)
 
     def test_get_file_volume_snapshot_list(self):
         result = self.file.get_file_volume_snapshot_list(100)
@@ -166,6 +193,24 @@ class FileTests(testing.TestCase):
             identifier=1234,
         )
 
+    def test_get_replication_partners(self):
+        self.file.get_replication_partners(1234)
+
+        self.assert_called_with(
+            'SoftLayer_Network_Storage',
+            'getReplicationPartners',
+            identifier=1234,
+        )
+
+    def test_get_replication_locations(self):
+        self.file.get_replication_locations(1234)
+
+        self.assert_called_with(
+            'SoftLayer_Network_Storage',
+            'getValidReplicationTargetDatacenterLocations',
+            identifier=1234,
+        )
+
     def test_delete_snapshot(self):
         result = self.file.delete_snapshot(100)
 
@@ -177,6 +222,25 @@ class FileTests(testing.TestCase):
             'deleteObject',
             identifier=100)
 
+    def test_order_file_volume_invalid_location(self):
+        mock = self.set_mock('SoftLayer_Location_Datacenter', 'getDatacenters')
+        mock.return_value = []
+
+        exception = self.assertRaises(
+            exceptions.SoftLayerError,
+            self.file.order_file_volume,
+            "performance_storage_nfs",
+            "dal05",
+            100,
+            None,
+            iops=100,
+        )
+
+        self.assertEqual(str(exception), "Invalid datacenter name "
+                                         "specified. Please provide the "
+                                         "lower case short name "
+                                         "(e.g.: dal09)")
+
     def test_order_file_volume_no_package(self):
         mock = self.set_mock('SoftLayer_Product_Package', 'getAllObjects')
         mock.return_value = []
@@ -187,7 +251,7 @@ class FileTests(testing.TestCase):
             "performance_storage_nfs",
             "dal05",
             40,
-            "LINUX",
+            None,
             iops=100,
         )
 
@@ -201,9 +265,38 @@ class FileTests(testing.TestCase):
             "performance_storage_nfs",
             "dal05",
             40,
+            None,
+            iops=100,
+        )
+
+    def test_order_file_volume_invalid_storage_type(self):
+        mock = self.set_mock('SoftLayer_Product_Package', 'getAllObjects')
+        mock.return_value = [{}]
+
+        exception = self.assertRaises(
+            exceptions.SoftLayerError,
+            self.file.order_file_volume,
+            "something_completely_different",
+            "dal05",
+            100,
+            None,
+            iops=100,
+        )
+        self.assertEqual(str(exception), "File volume storage_type must be "
+                                         "either Performance or Endurance")
+
+    def test_order_file_volume_os_type_provided(self):
+        exception = self.assertRaises(
+            exceptions.SoftLayerError,
+            self.file.order_file_volume,
+            "performance_storage_nfs",
+            "dal05",
+            100,
             "LINUX",
             iops=100,
         )
+        self.assertEqual(str(exception), "OS type is not used on file "
+                                         "storage orders")
 
     def test_order_file_volume_performance(self):
         mock = self.set_mock('SoftLayer_Product_Package', 'getAllObjects')
@@ -246,7 +339,7 @@ class FileTests(testing.TestCase):
             "performance_storage_nfs",
             "dal05",
             100,
-            "LINUX",
+            None,
             iops=100,
             )
 
@@ -330,7 +423,7 @@ class FileTests(testing.TestCase):
             "storage_service_enterprise",
             "dal05",
             100,
-            "LINUX",
+            None,
             tier_level=0.25,
             )
 
@@ -417,7 +510,7 @@ class FileTests(testing.TestCase):
             "storage_service_enterprise",
             "dal05",
             100,
-            "LINUX",
+            None,
             tier_level=0.25,
             snapshot_size=10,
             )
@@ -467,6 +560,78 @@ class FileTests(testing.TestCase):
             False,
         )
 
+    def test_order_snapshot_space_invalid_category(self):
+        mock = self.set_mock('SoftLayer_Product_Package', 'getAllObjects')
+        mock.return_value = [{
+            'id': 240,
+            'name': 'Endurance',
+            'items': [{
+                'capacity': '0',
+            }, {
+                'capacity': '5',
+                'prices': [{
+                    'locationGroupId': '530',
+                    }],
+            }, {
+                'capacity': '5',
+                'prices': [{
+                    'locationGroupId': '',
+                    'categories': [{
+                        'categoryCode': 'storage_file',
+                    }],
+                }],
+            }, {
+                'capacity': '5',
+                'prices': [{
+                    'locationGroupId': '',
+                    'categories': [{
+                        'categoryCode': 'storage_snapshot_space',
+                    }],
+                    'capacityRestrictionMinimum': '300',
+                }],
+            }, {
+                'capacity': '5',
+                'prices': [{
+                    'locationGroupId': '',
+                    'categories': [{
+                        'categoryCode': 'storage_snapshot_space',
+                    }],
+                    'capacityRestrictionMinimum': '100',
+                    'capacityRestrictionMaximum': '100',
+                }],
+            }, {
+                'capacity': '5',
+                'prices': [{
+                    'id': 46130,
+                    'locationGroupId': '',
+                    'categories': [{
+                        'categoryCode': 'storage_snapshot_space',
+                    }],
+                    'capacityRestrictionMinimum': '200',
+                    'capacityRestrictionMaximum': '200',
+                }],
+            }],
+        }]
+
+        billing_item_mock = self.set_mock('SoftLayer_Network_Storage',
+                                          'getObject')
+        billing_item_mock.return_value = {
+            'billingItem': {
+                'categoryCode': 'not_storage_service_enterprise'
+            }
+        }
+
+        exception = self.assertRaises(
+            exceptions.SoftLayerError,
+            self.file.order_snapshot_space,
+            100,
+            5,
+            None,
+            False,
+        )
+        self.assertEqual(str(exception), "File volume storage_type must be "
+                                         "Endurance")
+
     def test_order_snapshot_space(self):
         mock = self.set_mock('SoftLayer_Product_Package', 'getAllObjects')
         mock.return_value = [{
@@ -764,3 +929,167 @@ class FileTests(testing.TestCase):
                     'setupFee': '1'}],
                 },
             )
+
+    def test_order_file_duplicate_performance_no_duplicate_snapshot(self):
+        mock = self.set_mock('SoftLayer_Product_Package', 'getAllObjects')
+        mock.return_value = [fixtures.SoftLayer_Product_Package.SAAS_PACKAGE]
+
+        mock_volume = fixtures.SoftLayer_Network_Storage.DUPLICATABLE_VOLUME
+        prev_storage_type_keyname = mock_volume['storageType']['keyName']
+        mock_volume['storageType']['keyName'] = 'PERFORMANCE_FILE_STORAGE'
+        mock = self.set_mock('SoftLayer_Network_Storage', 'getObject')
+        mock.return_value = mock_volume
+
+        result = self.file.order_duplicate_volume(
+            102,
+            duplicate_snapshot_size=0)
+
+        self.assertEqual(fixtures.SoftLayer_Product_Order.placeOrder, result)
+
+        self.assert_called_with(
+            'SoftLayer_Product_Order',
+            'placeOrder',
+            args=({
+                'complexType': 'SoftLayer_Container_Product_Order_'
+                               'Network_Storage_AsAService',
+                'packageId': 759,
+                'prices': [
+                    {'id': 189433},
+                    {'id': 189453},
+                    {'id': 189993},
+                    {'id': 190053}
+                ],
+                'volumeSize': 500,
+                'quantity': 1,
+                'location': 449500,
+                'duplicateOriginVolumeId': 102,
+                'iops': 1000
+            },))
+
+        mock_volume['storageType']['keyName'] = prev_storage_type_keyname
+
+    def test_order_file_duplicate_performance(self):
+        mock = self.set_mock('SoftLayer_Product_Package', 'getAllObjects')
+        mock.return_value = [fixtures.SoftLayer_Product_Package.SAAS_PACKAGE]
+
+        mock_volume = fixtures.SoftLayer_Network_Storage.DUPLICATABLE_VOLUME
+        prev_storage_type_keyname = mock_volume['storageType']['keyName']
+        mock_volume['storageType']['keyName'] = 'PERFORMANCE_FILE_STORAGE'
+        mock = self.set_mock('SoftLayer_Network_Storage', 'getObject')
+        mock.return_value = mock_volume
+
+        result = self.file.order_duplicate_volume(
+            102,
+            origin_snapshot_id=470,
+            duplicate_size=1000,
+            duplicate_iops=2000,
+            duplicate_tier_level=None,
+            duplicate_snapshot_size=10
+            )
+
+        self.assertEqual(fixtures.SoftLayer_Product_Order.placeOrder, result)
+
+        self.assert_called_with(
+            'SoftLayer_Product_Order',
+            'placeOrder',
+            args=({
+                'complexType': 'SoftLayer_Container_Product_Order_'
+                               'Network_Storage_AsAService',
+                'packageId': 759,
+                'prices': [
+                    {'id': 189433},
+                    {'id': 189453},
+                    {'id': 190113},
+                    {'id': 190173},
+                    {'id': 191193}
+                ],
+                'volumeSize': 1000,
+                'quantity': 1,
+                'location': 449500,
+                'duplicateOriginVolumeId': 102,
+                'duplicateOriginSnapshotId': 470,
+                'iops': 2000
+            },))
+
+        mock_volume['storageType']['keyName'] = prev_storage_type_keyname
+
+    def test_order_file_duplicate_endurance_no_duplicate_snapshot(self):
+        mock = self.set_mock('SoftLayer_Product_Package', 'getAllObjects')
+        mock.return_value = [fixtures.SoftLayer_Product_Package.SAAS_PACKAGE]
+
+        mock_volume = fixtures.SoftLayer_Network_Storage.DUPLICATABLE_VOLUME
+        prev_storage_type_keyname = mock_volume['storageType']['keyName']
+        mock_volume['storageType']['keyName'] = 'ENDURANCE_FILE_STORAGE'
+        mock = self.set_mock('SoftLayer_Network_Storage', 'getObject')
+        mock.return_value = mock_volume
+
+        result = self.file.order_duplicate_volume(
+            102,
+            duplicate_snapshot_size=0)
+
+        self.assertEqual(fixtures.SoftLayer_Product_Order.placeOrder, result)
+
+        self.assert_called_with(
+            'SoftLayer_Product_Order',
+            'placeOrder',
+            args=({
+                'complexType': 'SoftLayer_Container_Product_Order_'
+                               'Network_Storage_AsAService',
+                'packageId': 759,
+                'prices': [
+                    {'id': 189433},
+                    {'id': 189453},
+                    {'id': 193433},
+                    {'id': 193373}
+                ],
+                'volumeSize': 500,
+                'quantity': 1,
+                'location': 449500,
+                'duplicateOriginVolumeId': 102
+            },))
+
+        mock_volume['storageType']['keyName'] = prev_storage_type_keyname
+
+    def test_order_file_duplicate_endurance(self):
+        mock = self.set_mock('SoftLayer_Product_Package', 'getAllObjects')
+        mock.return_value = [fixtures.SoftLayer_Product_Package.SAAS_PACKAGE]
+
+        mock_volume = fixtures.SoftLayer_Network_Storage.DUPLICATABLE_VOLUME
+        prev_storage_type_keyname = mock_volume['storageType']['keyName']
+        mock_volume['storageType']['keyName'] = 'ENDURANCE_FILE_STORAGE'
+        mock = self.set_mock('SoftLayer_Network_Storage', 'getObject')
+        mock.return_value = mock_volume
+
+        result = self.file.order_duplicate_volume(
+            102,
+            origin_snapshot_id=470,
+            duplicate_size=1000,
+            duplicate_iops=None,
+            duplicate_tier_level=4,
+            duplicate_snapshot_size=10
+            )
+
+        self.assertEqual(fixtures.SoftLayer_Product_Order.placeOrder, result)
+
+        self.assert_called_with(
+            'SoftLayer_Product_Order',
+            'placeOrder',
+            args=({
+                'complexType': 'SoftLayer_Container_Product_Order_'
+                               'Network_Storage_AsAService',
+                'packageId': 759,
+                'prices': [
+                    {'id': 189433},
+                    {'id': 189453},
+                    {'id': 194763},
+                    {'id': 194703},
+                    {'id': 194943}
+                ],
+                'volumeSize': 1000,
+                'quantity': 1,
+                'location': 449500,
+                'duplicateOriginVolumeId': 102,
+                'duplicateOriginSnapshotId': 470
+            },))
+
+        mock_volume['storageType']['keyName'] = prev_storage_type_keyname
